package ch.epfl.compnet.reliabletransfer;

import java.io.*;
import java.net.*;

/**
 * This class implements a receiver of the TP6 reliable transfer protocol
 * @author Laurent Pages
 * @author Marcin Poturalski
 */
public class ReliableServer {

	private DatagramSocket socket;
	private int nextPacketToDeliver;

	public ReliableServer(int port) {
		// Open the UDP socket.
        try {
            this.socket = new DatagramSocket(port);
            this.nextPacketToDeliver = 0;
        } catch (SocketException e) {
			e.printStackTrace();
		}
    }
    public void listen() {
		try {
			while (true) {
                // Allocate enough space for the biggest possible packet
				DatagramPacket dp = new DatagramPacket(new byte[MyDataPacket.MAX_SIZE],
                                                       MyDataPacket.MAX_SIZE);
				MyDataPacket packet;

                // Receive the new packet
				socket.receive(dp);
				packet = new MyDataPacket(dp.getData(), dp.getLength());

				// If we expect this packet, send data to upper layer.
                // In this case, we simply print the bytes
				if (packet.getSeqNum() == this.nextPacketToDeliver) {
                    System.out.print(new String(packet.getData()));
                    System.out.flush();
					nextPacketToDeliver++;
				}

				// ACK packet if it is the one we have been expecting.
                // Otherwise, do nothing (we drop the packet silently)
				if (packet.getSeqNum() < this.nextPacketToDeliver) {
					MyAckPacket ackp = new MyAckPacket(packet.getSeqNum());
					byte[] ackb = ackp.toByteArray();
					DatagramPacket ackdp = new DatagramPacket(ackb, ackb.length, dp.getAddress(), dp.getPort());

					socket.send(ackdp);
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
